;;---------------------=={  Incremental Numbering Suite  }==---------------------;;
;;                                                                               ;;
;;  Incremental Numbering Suite enables the user to dynamically place            ;;
;;  incrementing alphabetical or numerical text in a drawing, with a range of    ;;
;;  positioning utilities and an optional prefix and/or suffix.                  ;;
;;                                                                               ;;
;;  The sequential text can be created using Text, MText or Attributed Blocks;   ;;
;;  furthermore, the style and formatting of these objects can be altered        ;;
;;  directly from the main dialog, with all settings remembered between drawing  ;;
;;  sessions.                                                                    ;;
;;                                                                               ;;
;;  The user can modify the Text or Block layer, choose from a list of           ;;
;;  available Text Styles in the drawing, alter the Text or MText alignment,     ;;
;;  and change the Text Height or use the height defined by the selected         ;;
;;  Text Style. If MText is selected, the user may also toggle the use of an     ;;
;;  MText Background Mask.                                                       ;;
;;                                                                               ;;
;;  The user can enter optional Prefix, Middle and Suffix text, and have the     ;;
;;  option to increment either section, with the ability to increment            ;;
;;  alphabetical text and use decimals and leading zeros. The user can also      ;;
;;  specify any numerical increment, positive or negative.                       ;;
;;                                                                               ;;
;;  If the user has chosen to use Text or MText objects to house the             ;;
;;  incremental text, there is an option to enclose such objects with a border.  ;;
;;  The border may be Circular, Rectangular, Slot or an n-sided Polygon;         ;;
;;  created on a layer chosen from the main dialog.                              ;;
;;                                                                               ;;
;;  The size of the border may be controlled using an Offset from the Text or    ;;
;;  MText object; alternatively, the user may specify a fixed border size, with  ;;
;;  the option to pick either dimension from the drawing.                        ;;
;;                                                                               ;;
;;  Dynamic Mode                                                                 ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  Dynamic Mode is activated by enabling the option: 'Text Follows Cursor'.     ;;
;;                                                                               ;;
;;  This mode will display a real-time preview of the Text, MText or Attributed  ;;
;;  Block with any border that may be specified.                                 ;;
;;                                                                               ;;
;;  Note: As a result of the method used to display the real-time preview,       ;;
;;  Dynamic Mode restricts the use of standard AutoCAD functionality such as     ;;
;;  Object Snap, Orthomode, Tracking etc. To enable such functionality, uncheck  ;;
;;  the 'Text Follows Cursor' option at the top-left corner of the dialog.       ;;
;;                                                                               ;;
;;  Various positioning controls displayed at the command line:                  ;;
;;                                                                               ;;
;;  Dynamic Mode Placement Controls:                                             ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  [ Enter ]  -  (or Space/Right-Click) Exit Program [Cancel]                   ;;
;;  [ Click ]  -  Place Object                                                   ;;
;;  [   <   ]  -  Rotate Object Counter Clockwise                                ;;
;;  [   >   ]  -  Rotate Object Clockwise                                        ;;
;;  [   O   ]  -  Specify Object Rotation                                        ;;
;;  [  Tab  ]  -  Rotate Object by 90                                           ;;
;;  [   M   ]  -  Mirror Object Rotation                                         ;;
;;  [   C   ]  -  Align Object to Curve                                          ;;
;;  [   R   ]  -  Replace Existing Text/Attribute String                         ;;
;;  [   T   ]  -  Toggle Counter Increment                                       ;;
;;  [   B   ]  -  Rotate Polygonal Border                                        ;;
;;  [   A   ]  -  Toggle MText Background Mask                                   ;;
;;                                                                               ;;
;;  Align Object to Curve                                                        ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  The user can choose to align the object to a selected curve object           ;;
;;  (Line, LWPolyline, Polyline, XLine, Spline, Arc, Circle, Ellipse etc.) by    ;;
;;  pressing C or c during placement.                                            ;;
;;                                                                               ;;
;;  The user is then prompted to select a curve to which the text will be        ;;
;;  aligned. The text will follow the selected curve with various positioning    ;;
;;  controls available at the command-line:                                      ;;
;;                                                                               ;;
;;  Curve Alignment Controls                                                     ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  [ Enter ]  -  (or Space/Right-Click) Exit Curve Alignment [Cancel]           ;;
;;  [ Click ]  -  Place Object                                                   ;;
;;  [  +/-  ]  -  Increase/Decrease Object Offset                                ;;
;;  [   O   ]  -  Specify Object Offset                                          ;;
;;  [   P   ]  -  Toggle Object Perpendicularity                                 ;;
;;  [   B   ]  -  Rotate Polygonal Border                                        ;;
;;  [   A   ]  -  Toggle MText Background Mask                                   ;;
;;                                                                               ;;
;;  Replace Existing Text or Attribute String                                    ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  Upon pressing R or r during placement, the user is continuously prompted to  ;;
;;  select either Text, MText or Attribute, which, upon selection will be        ;;
;;  modified to contain the sequential text string.                              ;;
;;                                                                               ;;
;;  The user can exit this mode and return to standard text placement by         ;;
;;  pressing Enter, Space, or by Right-clicking the mouse at the prompt.         ;;
;;                                                                               ;;
;;  Standard Placement Mode                                                      ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  This mode is available when the 'Text Follows Cursor' is disabled            ;;
;;  (unticked).                                                                  ;;
;;                                                                               ;;
;;  When using this mode, there is no longer a real-time preview of the text at  ;;
;;  the cursor, however all standard AutoCAD functionality is available,         ;;
;;  (such as Object Snap, Tracking, Orthomode, etc.).                            ;;
;;                                                                               ;;
;;  The majority of placement controls are still available at the command-line:  ;;
;;                                                                               ;;
;;  Non-Dynamic Mode Placement Controls:                                         ;;
;;  --------------------------------------------                                 ;;
;;                                                                               ;;
;;  [ Enter ]  -  (or Space/Right-Click) Exit Program [Cancel]                   ;;
;;  [ Click ]  -  Place Object                                                   ;;
;;  [   O   ]  -  Specify Object Rotation                                        ;;
;;  [  RO   ]  -  Rotate Object by 90                                           ;;
;;  [   M   ]  -  Mirror Object Rotation                                         ;;
;;  [   C   ]  -  Align Object to Curve                                          ;;
;;  [   R   ]  -  Replace Existing Text/Attribute String                         ;;
;;  [   T   ]  -  Toggle Counter Increment                                       ;;
;;  [   B   ]  -  Rotate Polygonal Border                                        ;;
;;  [   A   ]  -  Toggle MText Background Mask                                   ;;
;;                                                                               ;;
;;-------------------------------------------------------------------------------;;
;;                                                                               ;;
;;  FUNCTION SYNTAX:  NumInc                                                     ;;
;;                                                                               ;;
;;-------------------------------------------------------------------------------;;
;;                                                                               ;;
;;  Author: Lee Mac, Copyright  2011 - www.lee-mac.com                          ;;
;;                                                                               ;;
;;-------------------------------------------------------------------------------;;
;;  Version:                                                                     ;;
;;                                                                               ;;
;;  1.0:  12/04/2009  -  First Release.                                          ;;
;;-------------------------------------------------------------------------------;;
;;  1.1:  14/04/2009  -  Added Prefix/Suffix Option.                             ;;
;;-------------------------------------------------------------------------------;;
;;  1.2:  15/04/2009  -  Added Dialog.                                           ;;
;;-------------------------------------------------------------------------------;;
;;  1.3:  15/04/2009  -  Added Option to Replace Existing text/attribute string. ;;
;;-------------------------------------------------------------------------------;;
;;  1.4:  18/04/2009  -  Made program compatible with leading zeros.             ;;
;;-------------------------------------------------------------------------------;;
;;  1.5:  16/06/2009  -  Upgraded program code.                                  ;;
;;-------------------------------------------------------------------------------;;
;;  1.6:  27/06/2009  -  Added Counter Toggle.                                   ;;
;;                    -  General Bug Fixes.                                      ;;
;;-------------------------------------------------------------------------------;;
;;  1.7:  18/02/2010  -  General Program Upgrade.                                ;;
;;-------------------------------------------------------------------------------;;
;;  1.8:  19/02/2010  -  Change Rotation Controls.                               ;;
;;-------------------------------------------------------------------------------;;
;;  1.9:  22/02/2010  -  Added option to not use GrRead loop, and hence allow    ;;
;;                       OSnap to function.                                      ;;
;;                    -  Added ability to place text in Table Cells.             ;;
;;-------------------------------------------------------------------------------;;
;;  2.0:  24/02/2010  -  Fixed Text Height Bug.                                  ;;
;;-------------------------------------------------------------------------------;;
;;  2.1:  05/05/2010  -  Removed imitation OSnap.                                ;;
;;                    -  Modified loop to allow multiple replacements when in    ;;
;;                       standard mode.                                          ;;
;;                    -  Added ability to use Alphabetical Increment.            ;;
;;-------------------------------------------------------------------------------;;
;;  2.2:  06/05/2010  -  Fixed bug with text case when incrementing alphabetical ;;
;;                       strings.                                                ;;
;;                    -  Added ability to border text with either Circle,        ;;
;;                       Rectangle or Slot; and offset from text.                ;;
;;-------------------------------------------------------------------------------;;
;;  2.3:  07/05/2010  -  Changed the way that layer/style globals are stored to  ;;
;;                       allow for layer/style changes between uses.             ;;
;;                    -  Removed Xref layers/styles from list.                   ;;
;;                    -  Fixed UCS bugs.                                         ;;
;;-------------------------------------------------------------------------------;;
;;  2.4:  10/05/2010  -  Added a 'By Style' option for text height selection.    ;;
;;                    -  Added option to enclose text with n-sided Polygon.      ;;
;;                    -  Added ability to rotate polygonal border.               ;;
;;-------------------------------------------------------------------------------;;
;;  2.5:  11/05/2010  -  Allowed for Zero height in TextStyle.                   ;;
;;                    -  Added ability to fix border size.                       ;;
;;                    -  Fixed Text Height variable bug.                         ;;
;;                    -  Fixed Rotation of odd-sided polygons.                   ;;
;;-------------------------------------------------------------------------------;;
;;  2.6:  12/05/2010  -  Fixed Rotation of odd-sided polygons when text is not   ;;
;;                       set to follow cursor.                                   ;;
;;                    -  Fixed Slot Bulges.                                      ;;
;;                    -  Added 'B' control to DCL About page.                    ;;
;;-------------------------------------------------------------------------------;;
;;  2.7:  22/05/2010  -  Fixed Border issue when in different view.              ;;
;;-------------------------------------------------------------------------------;;
;;  2.8:  24/05/2010  -  Changed DCL/CFG file save path to make program          ;;
;;                       compatible with Bricscad.                               ;;
;;-------------------------------------------------------------------------------;;
;;  2.9:  29/05/2010  -  Upgraded code to determine DCL/CFG Filepath, to allow   ;;
;;                       for AutoCAD Versions pre 2004.                          ;;
;;-------------------------------------------------------------------------------;;
;;  3.0:  10/10/2011  -  Program completely rewritten to improve program         ;;
;;                       performance, update code formatting and include the     ;;
;;                       following new features:                                 ;;
;;                    -  Ability to use Text, MText or an Attributed Block to    ;;
;;                       house incrementing string.                              ;;
;;                    -  Ability to change Text / MText Alignment.               ;;
;;                    -  Ability to toggle the use of a Background Mask with     ;;
;;                       MText.                                                  ;;
;;                    -  Ability to specify both dimensions for the fixed size   ;;
;;                       Slot / Rectangular border.                              ;;
;;                    -  Vastly improved alphabetical text incrementing.         ;;
;;                    -  Improved handling of negative numbers.                  ;;
;;                    -  Dialog interface completely redesigned to make it more  ;;
;;                       user-friendly and intuitive to navigate.                ;;
;;                    -  Improved non-dynamic mode interface & functionality     ;;
;;                    -  Program works in all UCS/Views correctly.               ;;
;;-------------------------------------------------------------------------------;;
;;  3.1:  11/10/2011  -  Fixed bug concerning null text size variable when       ;;
;;                       object is set to attributed block.                      ;;
;;-------------------------------------------------------------------------------;;

(setq NumIncVersion "3.1")

;;-------------------------------------------------------------------------------;;

(defun c:NumInc

    (
        /
        *error*
        _alignment
        _attachment
        _blocks
        _layers
        _styles
        acdoc
        acspc
        alignment
        alpha
        att-nme
        attachment
        attrib
        attribs
        blk-nme
        block
        blocks
        bor
        bor-enc
        bor-enc-fun
        bor-lay
        bor-rot
        bor-shp
        bor-shp-fun
        bor-sid
        bor-sid#
        bor-typ
        bor-typ-fun
        cfgfname
        create-bor
        create-obj
        dclflag
        dclfname
        dclid
        dcltitle
        deg
        dyn-flg
        elst
        ent
        file
        fix-ed1
        fix-ed1#
        fix-ed2
        fix-ed2#
        g1
        g2
        gr
        i
        inc-sec
        inc-str
        mid-str
        msg
        mtx-bak
        nm
        obj
        obj-typ
        obj-typ-fun
        off-ed1
        off-ed1#
        p1
        p2
        pre-str
        prop
        pt
        savepath
        ss
        string
        style
        suf-str
        symb
        symlist
        table
        tile
        tmp
        tog-cnt
        txt-aln
        txt-bst
        txt-lay
        txt-rot
        txt-sty
        txt-sty-fun
        txt-sze
        txt-sze#
        vallst
        varlst
        xa
    )

    (defun *error* ( msg )
        (if (eq "1" dyn-flg)
            (progn
                (if (and obj (not (vlax-erased-p obj))) (vla-delete obj))
                (if (and bor (not (vlax-erased-p bor))) (vla-delete bor))
            )
        )
        (if vallst (mapcar 'setvar varlst vallst))
        (if (and file (eq 'FILE (type file))) (setq file (close file)))
        (if (< 0 dclID) (setq dclID (unload_dialog dclID)))
        (if (not (wcmatch (strcase msg) "*BREAK,*CANCEL*,*EXIT*"))
            (princ (strcat "\nError: " msg))
        )
        (princ)
    )

    (setq varlst '("DIMZIN" "MODEMACRO")
          vallst  (mapcar 'getvar varlst)
    )

    (cond
        (   (= 4 (logand 4 (cdr (assoc 70 (tblsearch "LAYER" (getvar 'CLAYER))))))
            (princ "\nCurrent Layer Locked.")
        )
        (   (not (vl-file-directory-p (setq savepath (NumInc:GetSavePath))))
            (NumInc:Popup "Warning" 16 "Save Path not Valid.")
            (princ "\nSave Path not Valid.")
        )
        (   (progn
                (setq dclfname (strcat savepath "\\LMAC_NumInc_V" (vl-string-translate "." "-" NumIncVersion) ".dcl")
                      cfgfname (strcat savepath "\\LMAC_NumInc_V" (vl-string-translate "." "-" NumIncVersion) ".cfg")
                      dcltitle (strcat "Incremental Numbering Suite V" NumIncversion)
                )
                (not (NumInc:WriteDCL dclfname))
            )
            (NumInc:Popup "Warning" 16 "DCL File could not be Written.")
            (princ "\nDCL File could not be Written.")
        )
        (   (<= (setq dclID (load_dialog dclfname)) 0)
            (NumInc:Popup "Warning" 16 "The DCL File could not be Loaded.")
            (princ "\nThe DCL File could not be Loaded.")
        )
        (   t
            (setq SymList
                (list
                    (cons 'crv-per (/ pi 2.0))
                    (cons 'crv-off 0.0)
                    (cons 'mtx-bak :vlax-false)
                    (cons 'txt-rot 0.0)
                    (cons 'bor-rot nil)
                    (cons 'tog-cnt t)
                    (cons 'dyn-flg "1")
                    (cons 'pre-str  "")
                    (cons 'mid-str "1")
                    (cons 'suf-str  "")
                    (cons 'inc-str "1")
                    (cons 'inc-sec "inc-mid")
                    (cons 'obj-typ "obj-txt")
                    (cons 'blk-nme "")
                    (cons 'att-nme "")
                    (cons 'bor-enc "0")
                    (cons 'bor-shp "0")
                    (cons 'bor-sid "6")
                    (cons 'bor-lay (getvar 'CLAYER))
                    (cons 'bor-typ "bor-off")
                    (cons 'off-ed1 "1.0")
                    (cons 'fix-ed1 "1.0")
                    (cons 'fix-ed2 "1.0")
                    (cons 'txt-lay (getvar 'CLAYER))
                    (cons 'txt-sty (getvar 'TEXTSTYLE))
                    (cons 'txt-aln "Middle-Center")
                    (cons 'txt-bst "1")
                    (cons 'txt-sze
                        (rtos
                            (if
                                (zerop
                                    (cdr
                                        (assoc 40
                                            (setq style
                                                (tblsearch "STYLE" (getvar 'TEXTSTYLE))
                                            )
                                        )
                                    )
                                )
                                (cdr (assoc 42 style))
                                (cdr (assoc 40 style))
                            )
                        )
                    )
                )
            )
            (if (null (findfile cfgfname))
                (NumInc:WriteConfig cfgfname (mapcar 'cdr SymList))
            )
            (NumInc:ReadConfig cfgfname (mapcar 'car SymList))
            (foreach x SymList
                (if (null (boundp (car x)))
                    (set (car x) (cdr x))
                )
            )

            (setq _layers (NumInc:GetTableItems "LAYER")
                  _styles (NumInc:GetTableItems "STYLE")
                  _blocks (NumInc:GetBlockData)
            )

            (setq Alignment
                (list
                    (cons "Left"          acAlignmentLeft)
                    (cons "Center"        acAlignmentCenter)
                    (cons "Right"         acAlignmentRight)
                    (cons "Middle"        acAlignmentMiddle)
                    (cons "Top-Left"      acAlignmentTopLeft)
                    (cons "Top-Center"    acAlignmentTopCenter)
                    (cons "Top-Right"     acAlignmentTopRight)
                    (cons "Middle-Left"   acAlignmentMiddleLeft)
                    (cons "Middle-Center" acAlignmentMiddleCenter)
                    (cons "Middle-Right"  acAlignmentMiddleRight)
                    (cons "Bottom-Left"   acAlignmentBottomLeft)
                    (cons "Bottom-Center" acAlignmentBottomCenter)
                    (cons "Bottom-Right"  acAlignmentBottomRight)
                )
            )

            (setq Attachment
                (list
                    (cons "Top-Left"      acAttachmentPointTopLeft)
                    (cons "Top-Center"    acAttachmentPointTopCenter)
                    (cons "Top-Right"     acAttachmentPointTopRight)
                    (cons "Middle-Left"   acAttachmentPointMiddleLeft)
                    (cons "Middle-Center" acAttachmentPointMiddleCenter)
                    (cons "Middle-Right"  acAttachmentPointMiddleRight)
                    (cons "Bottom-Left"   acAttachmentPointBottomLeft)
                    (cons "Bottom-Center" acAttachmentPointBottomCenter)
                    (cons "Bottom-Right"  acAttachmentPointBottomRight)
                )
            )

            (setq _Alignment  (mapcar 'car Alignment))
            (setq _Attachment (mapcar 'car Attachment))
            
            (while (not (member dclflag '(1 0)))
                (cond
                    (   (not (new_dialog "numinc" dclID))
                        (NumInc:Popup "Warning" 16 "Dialog could not be Loaded.")
                        (princ "\nDialog could not be Loaded.")
                    )
                    (   t

                        ;;-------------------------------------------------------------------------------;;
                        ;;                                Top of Dialog                                  ;;
                        ;;-------------------------------------------------------------------------------;;
                     
                        (set_tile "dcltitle" dcltitle)

                        (set_tile "dyn-flg" dyn-flg)
                        (action_tile "dyn-flg" "(setq dyn-flg $value)")

                        ;;-------------------------------------------------------------------------------;;
                        ;;                                Top-Left Panel                                 ;;
                        ;;-------------------------------------------------------------------------------;;

                        (foreach symb '(pre-str mid-str suf-str inc-str)
                            (setq tile (strcase (vl-symbol-name symb) t))
                            (set_tile tile (eval symb))
                            (action_tile tile (strcat "(setq " tile " $value)"))
                        )

                        (set_tile inc-sec "1")
                        (action_tile "inc-pre" "(setq inc-sec $key)")
                        (action_tile "inc-mid" "(setq inc-sec $key)")
                        (action_tile "inc-suf" "(setq inc-sec $key)")
                     
                        ;;-------------------------------------------------------------------------------;;
                        ;;                              Bottom-Right Panel                               ;;
                        ;;-------------------------------------------------------------------------------;;
                     
                        (NumInc:MakeList "txt-lay" _layers)
                     
                        (set_tile "txt-lay"
                            (itoa
                                (cond
                                    (   (vl-position txt-lay _layers))
                                    (   (vl-position (setq txt-lay (getvar 'CLAYER)) _layers))
                                )
                            )
                        )
                        (action_tile "txt-lay" "(setq txt-lay (nth (atoi $value) _layers))")

                        ;;-------------------------------------------------------------------------------;;

                        (NumInc:MakeList "txt-sty" _styles)
                     
                        (set_tile "txt-sty"
                            (itoa
                                (cond
                                    (   (vl-position txt-sty _styles))
                                    (   (vl-position (setq txt-sty (getvar 'TEXTSTYLE)) _styles))
                                )
                            )
                        )
                        (
                            (setq txt-sty-fun
                                (lambda ( style / tmp )
                                    (if (zerop (setq tmp (cdr (assoc 40 (tblsearch "STYLE" style)))))
                                        (progn
                                            (set_tile  "txt-bst" (setq txt-bst "0"))
                                            (mode_tile "txt-bst"   1)
                                            (mode_tile "txt-sze"   0)
                                        )
                                        (progn
                                            (mode_tile "txt-bst" 0)
                                            (if (eq "1" txt-bst)
                                                (set_tile "txt-sze" (setq txt-sze (rtos tmp)))
                                            )
                                        )
                                    )
                                )
                            )
                            txt-sty
                        )
                        (action_tile "txt-sty" "(txt-sty-fun (setq txt-sty (nth (atoi $value) _styles)))")
                     
                        ;;-------------------------------------------------------------------------------;;
                     
                        (NumInc:MakeList "txt-aln" (if (eq "obj-mtx" obj-typ) _Attachment _Alignment))

                        (set_tile "txt-aln"
                            (itoa
                                (vl-position txt-aln
                                    (if (eq "obj-mtx" obj-typ) _Attachment _Alignment)
                                )
                            )
                        )

                        (action_tile "txt-aln"
                            (vl-prin1-to-string
                                (quote
                                    (setq txt-aln
                                        (nth (atoi $value) (if (eq "obj-mtx" obj-typ) _Attachment _Alignment))
                                    )
                                )
                            )
                        )

                        ;;-------------------------------------------------------------------------------;;

                        (set_tile    "txt-sze" txt-sze)
                        (action_tile "txt-sze" "(setq txt-sze $value)")

                        (if (eq "1" txt-bst)
                            (if (zerop (setq tmp (cdr (assoc 40 (tblsearch "STYLE" txt-sty)))))
                                (progn
                                    (set_tile  "txt-bst" (setq txt-bst "0"))
                                    (mode_tile "txt-bst"   1)
                                    (mode_tile "txt-sze"   0)
                                )
                                (progn
                                    (set_tile "txt-bst" txt-bst)
                                    (set_tile "txt-sze" (setq txt-sze (rtos tmp)))
                                )
                            )
                        )
                        (mode_tile "txt-sze" (atoi txt-bst))
                     
                        (action_tile "txt-bst"
                            (vl-prin1-to-string
                                (quote
                                    (progn
                                        (mode_tile "txt-sze" (atoi (setq txt-bst $value)))
                                        (if (eq "1" $value)
                                            (set_tile "txt-sze" (rtos (cdr (assoc 40 (tblsearch "STYLE" txt-sty)))))
                                        )
                                    )
                                )
                            )
                        )

                        ;;-------------------------------------------------------------------------------;;
                        ;;                               Bottom-Left Panel                               ;;
                        ;;-------------------------------------------------------------------------------;;

                        (set_tile "bor-enc" bor-enc)
                        (
                            (setq bor-enc-fun
                                (lambda ( value )
                                    (if (eq "1" value)
                                        (progn
                                            (mode_tile "bor-shp" 0)
                                            (if (eq "3" bor-shp) (mode_tile "bor-sid" 0))
                                            (mode_tile "bor-lay" 0)
                                            (mode_tile "bor-off" 0)
                                            (mode_tile "bor-fix" 0)
                                            (mode_tile "bor-pik" 0)
                                            (mode_tile "bor-ltx" 0)
                                            (if (eq "bor-off" bor-typ)
                                                (progn
                                                    (mode_tile "off-ed1" 0)
                                                    (mode_tile "fix-ed1" 1)
                                                    (mode_tile "fix-txt" 1)
                                                    (mode_tile "fix-ed2" 1)
                                                )
                                                (progn
                                                    (mode_tile "off-ed1" 1)
                                                    (mode_tile "fix-ed1" 0)
                                                    (if (member bor-shp '("1" "2"))
                                                        (progn
                                                            (mode_tile "fix-txt" 0)
                                                            (mode_tile "fix-ed2" 0)
                                                        )
                                                        (progn
                                                            (mode_tile "fix-txt" 1)
                                                            (mode_tile "fix-ed2" 1)
                                                        )
                                                    )
                                                )
                                            )
                                        )
                                        (foreach tile
                                           '(
                                                "bor-shp" "bor-sid" "bor-lay"
                                                "bor-off" "bor-fix" "off-ed1"
                                                "fix-ed1" "fix-ed2" "bor-pik"
                                                "fix-txt" "bor-ltx"
                                            )
                                            (mode_tile tile 1)
                                        )
                                    )
                                )
                            )
                            bor-enc
                        )
                        (action_tile "bor-enc" "(bor-enc-fun (setq bor-enc $value))")

                        ;;-------------------------------------------------------------------------------;;

                        (NumInc:MakeList "bor-shp" '("Circle" "Rectangle" "Slot" "Polygon"))

                        (set_tile "bor-shp" bor-shp)
                        (
                            (setq bor-shp-fun
                                (lambda ( value )
                                    (if (eq "bor-fix" bor-typ)
                                        (mapcar 'mode_tile '("bor-sid" "fix-txt" "fix-ed2")
                                            (cond
                                                (   (eq value "0")
                                                   '(1 1 1)
                                                )
                                                (   (member value '("1" "2"))
                                                   '(1 0 0)
                                                )
                                                (  '(0 1 1)
                                                )
                                            )
                                        )
                                        (mapcar 'mode_tile '("bor-sid" "fix-txt" "fix-ed2")
                                            (cond
                                                (   (member value '("0" "1" "2"))
                                                   '(1 1 1)
                                                )
                                                (  '(0 1 1)
                                                )
                                            )
                                        )
                                    )
                                )
                            )
                            bor-shp
                        )
                        (action_tile "bor-shp" "(bor-shp-fun (setq bor-shp $value))")

                        ;;-------------------------------------------------------------------------------;;

                        (set_tile "bor-sid" bor-sid)
                        (action_tile "bor-sid" "(setq bor-sid $value)")

                        ;;-------------------------------------------------------------------------------;;

                        (NumInc:MakeList "bor-lay" _layers)
                     
                        (set_tile "bor-lay"
                            (itoa
                                (cond
                                    (   (vl-position bor-lay _layers))
                                    (   (vl-position (setq bor-lay (getvar 'CLAYER)) _layers))
                                )
                            )
                        )
                        (action_tile "bor-lay" "(setq bor-lay (nth (atoi $value) _layers))")

                        ;;-------------------------------------------------------------------------------;;

                        (set_tile bor-typ "1")
                        (
                            (setq bor-typ-fun
                                (lambda ( typ )
                                    (if (eq "1" bor-enc)
                                        (if (eq "bor-off" typ)
                                            (mapcar 'mode_tile '("off-ed1" "fix-ed1" "fix-ed2" "fix-txt") '(0 1 1 1))
                                            (progn
                                                (mode_tile "off-ed1" 1)
                                                (mode_tile "fix-ed1" 0)
                                                (if (member bor-shp '("1" "2"))
                                                    (progn
                                                        (mode_tile "fix-ed2" 0)
                                                        (mode_tile "fix-txt" 0)
                                                    )
                                                )
                                            )
                                        )
                                    )
                                )
                            )
                            bor-typ
                        )
                        (action_tile "bor-off" "(bor-typ-fun (setq bor-typ $key))")
                        (action_tile "bor-fix" "(bor-typ-fun (setq bor-typ $key))")

                        ;;-------------------------------------------------------------------------------;;

                        (foreach symb '(off-ed1 fix-ed1 fix-ed2)
                            (setq tile (strcase (vl-symbol-name symb) t))
                            (set_tile tile (eval symb))
                            (action_tile tile (strcat "(setq " tile " $value)"))
                        )

                        (action_tile "bor-pik" "(done_dialog 3)")

                        ;;-------------------------------------------------------------------------------;;
                        ;;                                Top-Right Panel                                ;;
                        ;;-------------------------------------------------------------------------------;;

                        (set_tile obj-typ "1")
                     
                        (
                            (setq obj-typ-fun
                                (lambda ( typ )
                                    (if (eq typ "obj-blk")
                                        (progn
                                            (set_tile "lay-txt" "Block Layer: ")
                                            (foreach pair
                                               '(
                                                    ("blk-nme" 0)
                                                    ("blk-txt" 0)
                                                    ("blk-pik" 0)
                                                    ("att-txt" 0)
                                                    ("att-nme" 0)
                                                    ("sty-txt" 1)
                                                    ("txt-sty" 1)
                                                    ("aln-txt" 1)
                                                    ("txt-aln" 1)
                                                    ("txt-bst" 1)
                                                    ("txt-sze" 1)
                                                    ("bor-enc" 1)
                                                    ("bor-shp" 1)
                                                    ("bor-sid" 1)
                                                    ("bor-ltx" 1)
                                                    ("bor-lay" 1)
                                                    ("bor-off" 1)
                                                    ("bor-fix" 1)
                                                    ("off-ed1" 1)
                                                    ("fix-ed1" 1)
                                                    ("fix-ed2" 1)
                                                    ("bor-pik" 1)
                                                )
                                                (apply 'mode_tile pair)
                                            )
                                        )
                                        (progn
                                            (set_tile "lay-txt" "Text Layer: ")
                                            (foreach pair
                                               '(
                                                    ("blk-txt" 1)
                                                    ("blk-nme" 1)
                                                    ("blk-pik" 1)
                                                    ("att-txt" 1)
                                                    ("att-nme" 1)
                                                    ("sty-txt" 0)
                                                    ("txt-sty" 0)
                                                    ("aln-txt" 0)
                                                    ("txt-aln" 0)
                                                    ("bor-enc" 0)
                                                )
                                                (apply 'mode_tile pair)
                                            )
                                            (bor-enc-fun bor-enc)
                                            (txt-sty-fun txt-sty)
                                            (NumInc:MakeList "txt-aln" (if (eq "obj-mtx" obj-typ) _Attachment _Alignment))
                                            (set_tile "txt-aln"
                                                (itoa
                                                    (vl-position txt-aln
                                                        (if (eq "obj-mtx" obj-typ) _Attachment _Alignment)
                                                    )
                                                )
                                            )
                                        )
                                    )
                                )
                            )
                            obj-typ
                        )
                     
                        (action_tile "obj-txt" "(obj-typ-fun (setq obj-typ $key))")
                        (action_tile "obj-mtx" "(obj-typ-fun (setq obj-typ $key))")
                        (action_tile "obj-blk" "(obj-typ-fun (setq obj-typ $key))")

                        (if _blocks
                            (progn
                                (NumInc:MakeList "blk-nme" (setq blocks (mapcar 'car _blocks)))
                                (set_tile "blk-nme"
                                    (setq block
                                        (itoa
                                            (cond
                                                (   (vl-position blk-nme blocks))
                                                (   (setq blk-nme (car blocks))
                                                    0
                                                )
                                            )
                                        )
                                    )
                                )
                                (NumInc:MakeList "att-nme" (setq attribs (cdr (nth (atoi block) _blocks))))
                                (set_tile "att-nme"
                                    (setq attrib
                                        (itoa
                                            (cond
                                                (   (vl-position att-nme attribs))
                                                (   (setq att-nme (car attribs))
                                                    0
                                                )
                                            )
                                        )
                                    )
                                )
                            )
                            (foreach tile '("obj-blk" "blk-nme" "att-nme" "blk-pik")
                                (mode_tile tile 1)
                            )
                        )

                        (action_tile "blk-nme"
                            (vl-prin1-to-string
                                (quote
                                    (progn
                                        (setq blk-itm (nth (atoi (setq block $value)) _blocks)
                                              blk-nme (car blk-itm)
                                        )
                                        (NumInc:MakeList "att-nme" (setq attribs (cdr blk-itm)))
                                        (set_tile "att-nme"
                                            (setq attrib
                                                (itoa
                                                    (cond
                                                        (   (vl-position att-nme attribs))
                                                        (   (setq att-nme (car attribs))
                                                            0
                                                        )
                                                    )
                                                )
                                            )
                                        )
                                    )
                                )
                            )
                        )

                        (action_tile "blk-pik" "(done_dialog 2)")
                        (action_tile "att-nme" "(setq attrib $value att-nme (nth (atoi $value) attribs))")

                        ;;-------------------------------------------------------------------------------;;
                        ;;                                Base of Dialog                                 ;;
                        ;;-------------------------------------------------------------------------------;;

                        (action_tile "about" "(NumInc:About dclID)")

                        (action_tile "accept"
                            (vl-prin1-to-string
                                (quote
                                    (progn
                                        (if (eq "" inc-str) (setq inc-str "0"))
                                        (if (eq "" txt-sze) (setq txt-sze (rtos (getvar 'TEXTSIZE))))

                                        (setq alpha
                                            (or
                                                (and (eq "inc-pre" inc-sec) (not (distof pre-str 2)))
                                                (and (eq "inc-mid" inc-sec) (not (distof mid-str 2)))
                                                (and (eq "inc-suf" inc-sec) (not (distof suf-str 2)))
                                            )
                                        )
                                        (cond
                                            (   (and
                                                    (eq "1" bor-enc)
                                                    (eq "bor-off" bor-typ)
                                                    (not (setq off-ed1# (distof off-ed1)))
                                                )
                                                (NumInc:Popup "Information" 48 "Border Offset must be numerical.")
                                                (mode_tile "off-ed1" 2)
                                            )
                                            (   (and
                                                    (eq "1" bor-enc)
                                                    (eq "bor-off" bor-typ)
                                                    (< off-ed1# 0.0)
                                                )
                                                (NumInc:Popup "Information" 48 "Border Offset must be greater than zero.")
                                                (mode_tile "off-ed1" 2)
                                            )
                                            (   (and
                                                    (eq "1" bor-enc)
                                                    (eq "bor-fix" bor-typ)
                                                    (or
                                                        (not (setq fix-ed1# (distof fix-ed1)))
                                                        (and
                                                            (member bor-shp '("1" "2"))
                                                            (not (setq fix-ed2# (distof fix-ed2)))
                                                        )
                                                    )
                                                )
                                                (NumInc:Popup "Information" 48 "Border Size must be numerical.")
                                                (mode_tile "fix-ed1" 2)
                                            )
                                            (   (and
                                                    (eq "1" bor-enc)
                                                    (eq "bor-fix" bor-typ)
                                                    (or
                                                        (<= fix-ed1# 0.0)
                                                        (and
                                                            (member bor-shp '("1" "2"))
                                                            (<= fix-ed2# 0.0)
                                                        )
                                                    )
                                                )
                                                (NumInc:Popup "Information" 48 "Border Size must be greater than zero.")
                                                (mode_tile "fix-ed1" 2)
                                            )
                                            (   (not (distof inc-str 2))
                                                (NumInc:Popup "Information" 48 "Increment not numerical.")
                                                (mode_tile "inc-str" 2)
                                            )
                                            (   (and
                                                    (not (eq "obj-blk" obj-typ))
                                                    (not (setq txt-sze# (distof txt-sze)))
                                                )
                                                (NumInc:Popup "Information" 48 "Text Height not numerical.")
                                                (if (eq "0" txt-bst)
                                                    (mode_tile "txt-sze" 2)
                                                )
                                            )
                                            (   (and
                                                    (not (eq "obj-blk" obj-typ))
                                                    (<= txt-sze# 0.0)
                                                )
                                                (NumInc:Popup "Information" 48 "Text Height must be greater than zero.")
                                                (if (eq "0" txt-bst)
                                                    (mode_tile "txt-sze" 2)
                                                )
                                            )
                                            (   (and
                                                    (eq "1" bor-enc)
                                                    (eq "3" bor-shp)
                                                    (< (setq bor-sid# (atoi bor-sid)) 3)
                                                )
                                                (NumInc:Popup "Information" 48 "Number of Polygon Sides must be numerical\nand greater than 2.")
                                                (mode_tile "bor-sid" 2)
                                            )
                                            (   t
                                                (done_dialog 1)
                                            )
                                        )
                                    )
                                )
                            )
                        )

                        (setq dclflag (start_dialog))
                    )
                )
                (cond
                    (   (= 2 dclflag)
                        (while
                            (progn (setvar 'ERRNO 0) (setq ent (car (entsel "\nSelect Block: ")))
                                (cond
                                    (   (= 7 (getvar 'ERRNO))
                                        (princ "\nMissed, Try again.")
                                    )
                                    (   (eq 'ENAME (type ent))
                                        (if
                                            (and
                                                (eq "INSERT" (cdr (assoc 0 (setq elst (entget ent)))))
                                                (= 1 (cdr (assoc 66 elst)))
                                            )
                                            (progn
                                                (setq blk-nme
                                                    (if (vlax-property-available-p (setq obj (vlax-ename->vla-object ent)) 'effectivename)
                                                        (vla-get-effectivename obj)
                                                        (vla-get-name obj)
                                                    )
                                                )
                                                nil
                                            )
                                            (princ "\nPlease select a Block.")
                                        )
                                    )
                                )
                            )
                        )
                    )
                    (   (= 3 dclflag)
                        (cond
                            (   (eq "bor-off" bor-typ)
                                (setq off-ed1
                                    (cond
                                        (   (setq tmp
                                                (getdist
                                                    (strcat "\nSpecify Offset <" off-ed1 ">: ")
                                                )
                                            )
                                            (rtos tmp)
                                        )
                                        (   off-ed1   )
                                    )
                                )
                            )
                            (   t
                                (cond
                                    (   (member bor-shp '("0" "3"))
                                        (setq fix-ed1
                                            (cond
                                                (   (setq tmp
                                                        (getdist
                                                            (strcat "\nSpecify Border Radius <" fix-ed1 ">: ")
                                                        )
                                                    )
                                                    (rtos tmp)
                                                )
                                                (   fix-ed1   )
                                            )
                                        )
                                    )
                                    (   t
                                        (if
                                            (and
                                                (setq p1 (getpoint "\nSpecify First Point: "))
                                                (setq p2 (getcorner p1 "\nSpecify Opposite Corner: "))
                                            )
                                            (setq fix-ed1 (rtos (abs (- (car  p2) (car  p1))))
                                                  fix-ed2 (rtos (abs (- (cadr p2) (cadr p1))))
                                            )
                                        )
                                    )
                                )
                            )  
                        )
                    )
                )
            )
            (if (= 1 dclflag)
                (progn
                    (if
                        (setq ss
                            (ssget "_X"
                                (list '(0 . "ACAD_TABLE")
                                    (cons 410
                                        (if (= 1 (getvar 'CVPORT))
                                            (getvar 'CTAB)
                                            "Model"
                                        )
                                    )
                                )
                            )
                        )
                        (repeat (setq i (sslength ss))
                            (setq table (cons (vlax-ename->vla-object (ssname ss (setq i (1- i)))) table))
                        )
                    )
                    (setq acdoc (vla-get-activedocument (vlax-get-acad-object))
                          acspc (vlax-get-property acdoc (if (= 1 (getvar 'CVPORT)) 'paperspace 'modelspace))
                    )
                    (setq nm (trans '(0.0 0.0 1.0) 1 0 t)
                          xa (angle '(0.0 0.0 0.0) (trans (getvar 'UCSXDIR) 0 nm t))
                    )
                    (if (and (not (eq "obj-blk" obj-typ)) (NumInc:isAnnotative txt-sty))
                        (setq txt-sze# (/ txt-sze# (cond ((getvar 'CANNOSCALEVALUE)) (1.0))))
                    )
                    (setq symb
                        (nth
                            (vl-position inc-sec '("inc-pre" "inc-mid" "inc-suf"))
                           '(pre-str mid-str suf-str)
                        )
                    )
                    
                    (setq prop
                        (if
                            (and
                                (eq "obj-txt" obj-typ)
                                (not (eq "Left" txt-aln))
                            )
                            'TextAlignmentPoint
                            'InsertionPoint
                        )
                    )
                    
                    (setq create-obj
                        (cond
                            (   (eq "obj-txt" obj-typ)
                                (lambda ( point string / obj )
                                    (setq point (vlax-3D-point (trans point 1 0))
                                            obj (vla-addtext acspc string point txt-sze#)
                                    )
                                    (vla-put-stylename obj txt-sty)
                                    (vla-put-layer     obj txt-lay)
                                    (vla-put-alignment obj (cdr (assoc txt-aln Alignment)))
                                    (if (eq "Left" txt-aln)
                                        (vla-put-insertionpoint     obj point)
                                        (vla-put-textalignmentpoint obj point)
                                    )
                                    (vla-put-rotation  obj (+ xa txt-rot))
                                    obj
                                )
                            )
                            (   (eq "obj-mtx" obj-typ)
                                (lambda ( point string / obj )
                                    (setq point (vlax-3D-point (trans point 1 0)))
                                    (setq obj
                                        (vla-addmtext acspc point
                                            (
                                                (lambda ( box ) (- (caadr box) (caar box)))
                                                (textbox
                                                    (list
                                                        (cons 1  (strcat string "A"))
                                                        (cons 40 txt-sze#)
                                                        (cons 7  txt-sty)
                                                    )
                                                )
                                            )
                                            string
                                        )
                                    )
                                    (vla-put-stylename obj txt-sty)
                                    (vla-put-layer     obj txt-lay)
                                    (vla-put-height    obj txt-sze#)
                                    (vla-put-attachmentpoint obj (cdr (assoc txt-aln Attachment)))
                                    (vla-put-insertionpoint obj point)
                                    (vla-put-rotation obj  txt-rot)
                                    (vla-put-backgroundfill obj mtx-bak)
                                    obj
                                )
                            )
                            (   (eq "obj-blk" obj-typ)
                                (lambda ( point string / obj )
                                    (setq point (vlax-3D-point (trans point 1 0))
                                          obj   (vla-insertblock acspc point blk-nme 1.0 1.0 1.0 (+ xa txt-rot))
                                    )
                                    (vl-some
                                        (function
                                            (lambda ( attrib )
                                                (if (eq (vla-get-tagstring attrib) att-nme)
                                                    (null (vla-put-textstring attrib string))
                                                )
                                            )
                                        )
                                        (vlax-invoke obj 'getattributes)
                                    )
                                    (vla-put-layer obj txt-lay)
                                    obj
                                )
                            )
                        )
                    )

                    (setq create-bor
                        (lambda ( obj prop / bor )
                            (setq bor
                                (vlax-ename->vla-object
                                    (NumInc:CreateTextBorder
                                        (vlax-vla-object->ename obj) bor-shp
                                        (cond (off-ed1#) (0.0)) fix-ed1# fix-ed2# bor-sid#
                                    )
                                )
                            )
                            (vla-put-layer bor bor-lay)
                            (if (and (eq "3" bor-shp) bor-rot)
                                (vla-rotate bor (vlax-3D-point (NumInc:PolygonCentroid bor)) (/ pi bor-sid#))
                            )
                            bor
                        )
                    )
                    
                    (cond
                        (   (eq "1" dyn-flg)
                            (setq obj (create-obj (cadr (grread t 13 0)) (strcat pre-str mid-str suf-str)))

                            (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                (setq bor (create-bor obj prop))
                            )
                         
                            (princ
                                (setq msg
                                    (strcat
                                        "\n[C]urve Aligned, [R]eplace, R[o]tate CCW[<] / CW[>], [T]oggle Count\n"
                                        (if
                                            (and
                                                (not (eq "obj-blk" obj-typ))
                                                (eq "1" bor-enc)
                                                (eq "3" bor-shp)
                                            )
                                            "Rotate [B]order, " ""
                                        )
                                        "[Tab] = Rotate 90" (chr 186)
                                        ", [M]irror Rotation"
                                        (if (eq "obj-mtx" obj-typ) ", B[a]ckground Mask" "")
                                        " <Exit>"
                                    )
                                )
                            )
                            (setvar 'MODEMACRO
                                (strcat "Rotation: "
                                    (rtos (rem (+ 360.0 (* 180.0 (/ txt-rot pi))) 360) 2 2) (chr 186)
                                )
                            )
                            (while
                                (progn (setq gr (grread t 15 0) g1 (car gr) g2 (cadr gr))
                                    (cond
                                        (   (member g1 '(3 5))
                                            (setq p1 (vlax-3D-point (trans g2 1 0)))
                                            (if bor
                                                (vla-move bor (vlax-get-property obj prop) p1)
                                            )
                                            (vlax-put-property obj prop p1)

                                            (if (= 3 g1)
                                                (progn
                                                    (if (and table (NumInc:TextinCell table p1 (strcat pre-str mid-str suf-str)))
                                                        (progn
                                                            (vla-delete obj)
                                                            (if bor (vla-delete bor))
                                                        )
                                                    )
                                                    (if tog-cnt (NumInc:Increment symb inc-str alpha))
                                                    (setq obj (create-obj g2 (strcat pre-str mid-str suf-str)))

                                                    (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                                        (setq bor (create-bor obj prop))
                                                    )
                                                )
                                            )
                                            t
                                        )
                                        (   (= 25 g1)
                                            (vla-delete obj)
                                            (if bor (vla-delete bor))
                                            nil
                                        )
                                        (   (= 2 g1)
                                            (cond
                                                (   (member g2 '(67 99))  ;; C/c

                                                    (vla-delete obj)
                                                    (if bor (vla-delete bor))
                                                 
                                                    (while
                                                        (setq ent
                                                            (NumInc:SelectIf "\nSelect Curve <Exit>: "
                                                                (function
                                                                    (lambda ( x )
                                                                        (not
                                                                            (vl-catch-all-error-p
                                                                                (vl-catch-all-apply 'vlax-curve-getendparam (list x))
                                                                            )
                                                                        )
                                                                    )
                                                                )
                                                                entsel
                                                            )
                                                        )
                                                        (if (NumInc:AligntoCurve
                                                                (setq obj (create-obj (cadr ent) (strcat pre-str mid-str suf-str))) prop (car ent)
                                                                (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                                                    (setq bor (create-bor obj prop))
                                                                )
                                                            )
                                                            (if tog-cnt (NumInc:Increment symb inc-str alpha))
                                                            (progn
                                                                (vla-delete obj)
                                                                (if bor (vla-delete bor))
                                                            )
                                                        )
                                                    )

                                                    (setq obj (create-obj (cadr (grread t 13 0)) (strcat pre-str mid-str suf-str)))
                                                    (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                                        (setq bor (create-bor obj prop))
                                                    )
                                                    (princ msg)
                                                )
                                                (   (member g2 '(44 46 60 62))  ;; </>
                                                    (if (member g2 '(44 60))
                                                        (setq deg (/ pi  180.0))
                                                        (setq deg (/ pi -180.0))
                                                    )
                                                    (setvar 'MODEMACRO
                                                        (strcat "Rotation: "
                                                            (rtos
                                                                (rem
                                                                    (+ 360.0
                                                                        (* 180.0
                                                                            (/ (setq txt-rot (+ txt-rot deg)) pi)
                                                                        )
                                                                    )
                                                                    360
                                                                )
                                                                2 2
                                                            )
                                                            (chr 186)
                                                        )
                                                    )
                                                    (vla-put-rotation obj (+ (vla-get-rotation obj) deg))
                                                    (if bor
                                                        (vla-rotate bor (vlax-get-property obj prop) deg)
                                                    )
                                                    t
                                                )
                                                (   (member g2 '(79 111))  ;; O/o
                                                    (setq txt-rot
                                                        (cond
                                                            (
                                                                (getangle
                                                                    (strcat "\nSpecify "
                                                                        (cdr
                                                                            (assoc obj-typ
                                                                               '(
                                                                                    ("obj-txt" . "Text")
                                                                                    ("obj-mtx" . "MText")
                                                                                    ("obj-blk" . "Block")
                                                                                )
                                                                            )
                                                                        )
                                                                        " Rotation <" (angtos txt-rot) ">: "
                                                                    )
                                                                )
                                                            )
                                                            (   txt-rot   )
                                                        )
                                                    )
                                                    (setvar 'MODEMACRO
                                                        (strcat "Rotation: "
                                                            (rtos
                                                                (rem
                                                                    (+ 360.0 (* 180.0 (/ txt-rot pi)))
                                                                    360
                                                                )
                                                                2 2
                                                            )
                                                            (chr 186)
                                                        )
                                                    )
                                                    (if bor
                                                        (vla-rotate bor (vlax-get-property obj prop)
                                                            (- txt-rot
                                                                (if (eq "obj-mtx" obj-typ)
                                                                    (vla-get-rotation obj)
                                                                    (- (vla-get-rotation obj) xa)
                                                                )
                                                            )
                                                        )
                                                    )
                                                    (vla-put-rotation obj
                                                        ((lambda ( a ) (if (eq "obj-mtx" obj-typ) a (+ a xa))) txt-rot)
                                                    )
                                                    (princ msg)
                                                )
                                                (   (member g2 '(84 116))  ;; T/t
                                                    (if (setq tog-cnt (not tog-cnt))
                                                        (princ "\n<Counter Enabled>")
                                                        (princ "\n<Counter Disabled>")
                                                    )
                                                    (princ msg)
                                                )
                                                (   (member g2 '(66 98))  ;; B/b
                                                    (if
                                                        (and
                                                            (not (eq "obj-blk" obj-typ))
                                                            (eq "1" bor-enc)
                                                            (eq "3" bor-shp)
                                                            bor
                                                        )
                                                        (progn
                                                            (setq bor-rot (not bor-rot))
                                                            (vla-rotate bor
                                                                (vlax-3D-point (NumInc:PolygonCentroid bor))
                                                                (/ pi bor-sid#)
                                                            )
                                                        )
                                                        (princ (strcat "\nInvalid Keypress." msg))
                                                    )
                                                    t
                                                )
                                                (   (= g2 9)  ;; Tab
                                                    (setq txt-rot (rem (+ pi pi txt-rot) (+ pi pi)))
                                                 
                                                    (if
                                                        (vl-some
                                                            (function
                                                                (lambda ( x )
                                                                    (equal txt-rot x 1e-6)
                                                                )
                                                            )
                                                            (list (* pi 0.0) (* pi 0.5) (* pi 1.0) (* pi 1.5))
                                                        )
                                                        (setq txt-rot (rem (+ txt-rot (/ pi 2.0)) (+ pi pi)))
                                                        (setq txt-rot (NumInc:RoundTo txt-rot (/ pi 2.0)))
                                                    )
                                                    (setvar 'MODEMACRO
                                                        (strcat "Rotation: "
                                                            (rtos
                                                                (rem
                                                                    (+ 360.0 (* 180.0 (/ txt-rot pi)))
                                                                    360
                                                                )
                                                                2 2
                                                            )
                                                            (chr 186)
                                                        )
                                                    )
                                                    (if bor
                                                        (vla-rotate bor (vlax-get-property obj prop)
                                                            (- txt-rot
                                                                (if (eq "obj-mtx" obj-typ)
                                                                    (vla-get-rotation obj)
                                                                    (- (vla-get-rotation obj) xa)
                                                                )
                                                            )
                                                        )
                                                    )
                                                    (vla-put-rotation obj
                                                        ((lambda ( a ) (if (eq "obj-mtx" obj-typ) a (+ a xa))) txt-rot)
                                                    )
                                                    t
                                                )
                                                (   (member g2 '(77 109))  ;; M/m
                                                    (setq txt-rot (rem (+ pi pi (* -1.0 txt-rot)) (+ pi pi)))

                                                    (setvar 'MODEMACRO
                                                        (strcat "Rotation: "
                                                            (rtos
                                                                (rem
                                                                    (+ 360.0 (* 180.0 (/ txt-rot pi)))
                                                                    360
                                                                )
                                                                2 2
                                                            )
                                                            (chr 186)
                                                        )
                                                    )
                                                 
                                                    (if bor
                                                        (vla-rotate bor (vlax-get-property obj prop)
                                                            (- txt-rot
                                                                (if (eq "obj-mtx" obj-typ)
                                                                    (vla-get-rotation obj)
                                                                    (- (vla-get-rotation obj) xa)
                                                                )
                                                            )
                                                        )
                                                    )
                                                    (vla-put-rotation obj
                                                        ((lambda ( a ) (if (eq "obj-mtx" obj-typ) a (+ a xa))) txt-rot)
                                                    )
                                                    t
                                                )
                                                (   (member g2 '(82 114))  ;; R/r

                                                    (vla-delete obj)
                                                    (if bor (vla-delete bor))

                                                    (while
                                                        (setq ent
                                                            (car
                                                                (NumInc:SelectIf "\nSelect Text, MText or Attribute <Exit>: "
                                                                    (function
                                                                        (lambda ( x )
                                                                            (member (cdr (assoc 0 (entget x))) '("TEXT" "MTEXT" "ATTRIB"))
                                                                        )
                                                                    )
                                                                    nentsel
                                                                )
                                                            )
                                                        )
                                                        (if
                                                            (= 4
                                                                (logand 4
                                                                    (cdr
                                                                        (assoc 70
                                                                            (tblsearch "LAYER"
                                                                                (cdr (assoc 8 (entget ent)))
                                                                            )
                                                                        )
                                                                    )
                                                                )
                                                            )
                                                            (princ "\nObject on Locked Layer.")
                                                            (progn
                                                                (vla-put-textstring (vlax-ename->vla-object ent) (strcat pre-str mid-str suf-str))
                                                                (if tog-cnt (NumInc:Increment symb inc-str alpha))
                                                            )
                                                        )
                                                    )

                                                    (setq obj (create-obj (cadr (grread t 13 0)) (strcat pre-str mid-str suf-str)))
                                                    (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                                        (setq bor (create-bor obj prop))
                                                    )
                                                    (princ msg)
                                                )
                                                (   (member g2 '(65 97))  ;; A/a

                                                    (if (eq "obj-mtx" obj-typ)
                                                        (progn
                                                            (vlax-put obj 'backgroundfill
                                                                (setq mtx-bak (~ (vlax-get obj 'backgroundfill)))
                                                            )
                                                            (if (zerop mtx-bak)
                                                                (princ "\n<Background Mask Off>")
                                                                (princ "\n<Background Mask On>")
                                                            )
                                                        )
                                                        (princ "\nInvalid Keypress.")
                                                    )
                                                    (princ msg)
                                                )
                                                (   (member g2 '(13 32))  ;; Enter/Space
                                                 
                                                    (vla-delete obj)
                                                    (if bor (vla-delete bor))      
                                                    nil
                                                )
                                                (   (princ (strcat "\nInvalid Keypress." msg))   )
                                            )
                                        )
                                        (   t
                                            (vla-delete obj)
                                            (if bor (vla-delete bor))
                                            nil
                                        )
                                    )
                                )
                            )
                        )
                        (   t

                            (setq msg
                                (strcat
                                    "\nPick Point or [C]urve Aligned, [R]eplace, R[o]tation, [T]oggle Count\n"
                                    (if
                                        (and
                                            (not (eq "obj-blk" obj-typ))
                                            (eq "1" bor-enc)
                                            (eq "3" bor-shp)
                                        )
                                        "Rotate [B]order, " ""
                                    )
                                    "[Ro]tate 90" (chr 186) ", [M]irror Rotation"
                                    (if (eq "obj-mtx" obj-typ) ", B[a]ckground Mask" "")
                                    " <Exit>: "
                                )
                            )

                            (setvar 'MODEMACRO
                                (strcat "Rotation: "
                                    (rtos (rem (+ 360.0 (* 180.0 (/ txt-rot pi))) 360) 2 2) (chr 186)
                                )
                            )
                         
                            (while
                                (progn
                                    (initget
                                        (strcat
                                            "Curve Replace rOtation Toggle ROtate"
                                            (if
                                                (and
                                                    (not (eq "obj-blk" obj-typ))
                                                    (eq "1" bor-enc)
                                                    (eq "3" bor-shp)
                                                )
                                                " Border" ""
                                            )
                                            " Mirror"
                                            (if (eq "obj-mtx" obj-typ) " bAckground" "")
                                        )
                                    )
                                    (setq pt (getpoint msg))
                                    (cond
                                        (   (null pt)
                                            nil
                                        )
                                        (   (listp pt)
                                            (if
                                                (null
                                                    (and table
                                                        (NumInc:TextinCell table
                                                            (vlax-3D-point (trans pt 1 0)) (strcat pre-str mid-str suf-str)
                                                        )
                                                    )
                                                )
                                                (progn
                                                    (setq obj (create-obj pt (strcat pre-str mid-str suf-str)))
                                                    (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                                        (setq bor (create-bor obj prop))
                                                    )
                                                )
                                            )
                                            (if tog-cnt (NumInc:Increment symb inc-str alpha))
                                            (princ "\n--------------------")
                                            t
                                        )
                                        (   (eq "Curve" pt)
                                                  
                                            (while
                                                (setq ent
                                                    (NumInc:SelectIf "\nSelect Curve <Exit>: "
                                                        (function
                                                            (lambda ( x )
                                                                (not
                                                                    (vl-catch-all-error-p
                                                                        (vl-catch-all-apply 'vlax-curve-getendparam (list x))
                                                                    )
                                                                )
                                                            )
                                                        )
                                                        entsel
                                                    )
                                                )
                                                (if (NumInc:AligntoCurve
                                                        (setq obj (create-obj (cadr ent) (strcat pre-str mid-str suf-str))) prop (car ent)
                                                        (if (and (not (eq "obj-blk" obj-typ)) (eq "1" bor-enc))
                                                            (setq bor (create-bor obj prop))
                                                        )
                                                    )
                                                    (if tog-cnt (NumInc:Increment symb inc-str alpha))
                                                    (progn
                                                        (vla-delete obj)
                                                        (if bor (vla-delete bor))
                                                    )
                                                )
                                            )
                                            t
                                        )
                                        (   (eq "Replace" pt)

                                            (while
                                                (setq ent
                                                    (car
                                                        (NumInc:SelectIf "\nSelect Text, MText or Attribute <Exit>: "
                                                            (function
                                                                (lambda ( x )
                                                                    (member (cdr (assoc 0 (entget x))) '("TEXT" "MTEXT" "ATTRIB"))
                                                                )
                                                            )
                                                            nentsel
                                                        )
                                                    )
                                                )
                                                (if
                                                    (= 4
                                                        (logand 4
                                                            (cdr
                                                                (assoc 70
                                                                    (tblsearch "LAYER"
                                                                        (cdr (assoc 8 (entget ent)))
                                                                    )
                                                                )
                                                            )
                                                        )
                                                    )
                                                    (princ "\nObject on Locked Layer.")
                                                    (progn
                                                        (vla-put-textstring (vlax-ename->vla-object ent) (strcat pre-str mid-str suf-str))
                                                        (if tog-cnt (NumInc:Increment symb inc-str alpha))
                                                    )
                                                )
                                            )
                                            t
                                        )
                                        (   (eq "rOtation" pt)
                                         
                                            (setq txt-rot
                                                (cond
                                                    (
                                                        (getangle
                                                            (strcat "\nSpecify "
                                                                (cdr
                                                                    (assoc obj-typ
                                                                       '(
                                                                            ("obj-txt" . "Text")
                                                                            ("obj-mtx" . "MText")
                                                                            ("obj-blk" . "Block")
                                                                        )
                                                                    )
                                                                )
                                                                " Rotation <" (angtos txt-rot) ">: "
                                                            )
                                                        )
                                                    )
                                                    (   txt-rot   )
                                                )
                                            )
                                            (setvar 'MODEMACRO
                                                (strcat "Rotation: "
                                                    (rtos
                                                        (rem
                                                            (+ 360.0 (* 180.0 (/ txt-rot pi)))
                                                            360
                                                        )
                                                        2 2
                                                    )
                                                    (chr 186)
                                                )
                                            )
                                            t
                                        )
                                        (   (eq "Toggle" pt)
                                            (if (setq tog-cnt (not tog-cnt))
                                                (princ "\n<Counter Enabled>")
                                                (princ "\n<Counter Disabled>")
                                            )
                                            t
                                        )
                                        (   (eq "Border" pt)
                                            (princ "\n<Border Rotated>")
                                            (setq bor-rot (not bor-rot))
                                            t
                                        )
                                        (   (eq "ROtate" pt)
                                            (setq txt-rot (rem (+ pi pi txt-rot) (+ pi pi)))
                                            (if
                                                (vl-some
                                                    (function
                                                        (lambda ( x )
                                                            (equal txt-rot x 1e-6)
                                                        )
                                                    )
                                                    (list (* pi 0.0) (* pi 0.5) (* pi 1.0) (* pi 1.5))
                                                )
                                                (setq txt-rot (rem (+ txt-rot (/ pi 2.0)) (+ pi pi)))
                                                (setq txt-rot (NumInc:RoundTo txt-rot (/ pi 2.0)))
                                            )
                                            (princ
                                                (strcat "\n"
                                                    (setvar 'MODEMACRO
                                                        (strcat "Rotation: "
                                                            (rtos
                                                                (rem
                                                                    (+ 360.0 (* 180.0 (/ txt-rot pi)))
                                                                    360
                                                                )
                                                                2 2
                                                            )
                                                            (chr 186)
                                                        )
                                                    )
                                                )
                                            )
                                            t
                                        )
                                        (   (eq "Mirror" pt)
                                            (setq txt-rot (rem (+ pi pi (* -1.0 txt-rot)) (+ pi pi)))
                                            (princ
                                                (strcat "\n"
                                                    (setvar 'MODEMACRO
                                                        (strcat "Rotation: "
                                                            (rtos
                                                                (rem
                                                                    (+ 360.0 (* 180.0 (/ txt-rot pi)))
                                                                    360
                                                                )
                                                                2 2
                                                            )
                                                            (chr 186)
                                                        )
                                                    )
                                                )
                                            )
                                            t
                                        )
                                        (   (eq "bAckground" pt)
                                            (if (zerop (setq mtx-bak (~ mtx-bak)))
                                                (princ "\n<Background Mask Off>")
                                                (princ "\n<Background Mask On>")
                                            )
                                            t
                                        )
                                    )
                                )
                            )
                        )
                    )
                    (NumInc:WriteConfig cfgfname (mapcar 'eval (mapcar 'car SymList)))
                )
                (princ "\n*Cancel*")
            )
        )
    )
    (if (< 0 dclID) (setq dclID (unload_dialog dclID)))
    (mapcar 'setvar varlst vallst)
    (princ)
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:SelectIf ( msg pred func / ent )
    (setq pred (eval pred))
    (while
        (progn (setvar 'ERRNO 0) (setq ent (func msg))
            (cond
                (   (= 7 (getvar 'ERRNO))
                    (princ "\nMissed, try again.")
                )
                (   (eq 'ENAME (type (car ent)))
                    (if (and pred (null (pred (car ent))))
                        (princ "\nInvalid Object.")
                    )
                )
            )
        )
    )
    ent
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:isAnnotative ( style / object annotx )
    (and
        (setq object (tblobjname "STYLE" style))
        (setq annotx (cadr (assoc -3 (entget object '("AcadAnnotative")))))
        (= 1 (cdr (assoc 1070 (reverse annotx))))
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:GetBlockData ( / a b )
    (while (setq a (tblnext "BLOCK" (null a)))
        (if
            (and
                (null (wcmatch (cdr (assoc 2 a)) "`**,*|*"))
                (= 2 (logand 2 (cdr (assoc 70 a))))
            )
            (setq b
                (cons
                    (cons (cdr (assoc 2 a))
                        (
                            (lambda ( c / d e )
                                (while (setq c (entnext c))
                                    (if (eq "ATTDEF" (cdr (assoc 0 (setq d (entget c)))))
                                        (setq e (cons (strcase (cdr (assoc 2 d))) e))
                                    )
                                )
                                (acad_strlsort e)
                            )
                            (tblobjname "BLOCK" (cdr (assoc 2 a)))
                        )
                    )
                    b
                )
            )
        )
    )
    (vl-sort b '(lambda ( a b ) (< (car a) (car b))))
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:GetTableItems ( table / a b )
    (while (setq a (tblnext table (null a)))
        (if (null (wcmatch (cdr (assoc 2 a)) "`**,*|*"))
            (setq b (cons (cdr (assoc 2 a)) b))
        )
    )
    (acad_strlsort b)
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:RoundTo ( a b )
    (* b (fix (/ (+ a (* b (if (minusp a) -0.5 0.5))) b)))
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:About ( id / _dialogtext )

    (defun _dialogtext ( key str )
        (set_tile key str)
        (start_image key)
        (vector_image 0 (1- (dimy_tile key)) (dimx_tile key) (1- (dimy_tile key)) 0)
        (end_image)
    )
    
    (cond
        (   (not (new_dialog "about" id))
            (NumInc:Popup "Warning" 16 "\"About\" Dialog could not be Loaded.")
            (princ "\n\"About\" Dialog could not be Loaded.")
            (princ)
        )
        (   t
            (foreach pair
               '(
                    ("title1" "Incremental Numbering Suite")
                    ("title2" "Placement Controls")
                    ("title3" "Curve Alignment Controls")
                )
                (apply '_dialogtext pair)
            )
            (start_dialog)
        )
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:GetSavePath ( / tmp )
    (cond      
        (   (setq tmp (getvar 'ROAMABLEROOTPREFIX))
            (strcat (vl-string-right-trim "\\" (vl-string-translate "/" "\\" tmp)) "\\Support")
        )
        (   (setq tmp (findfile "ACAD.pat"))
            (vl-string-right-trim "\\" (vl-string-translate "/" "\\" (vl-filename-directory tmp)))
        )
        (   (vl-string-right-trim "\\" (vl-filename-directory (vl-filename-mktemp))))
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:WriteDCL ( fname / file )
    (cond
        (   (findfile fname)
        )
        (   (setq file (open fname "w"))
            (foreach line
               '(
                    "//------------=={ Incremental Numbering Suite }==-------------//"
                    "//                                                            //"
                    "//  NumInc.dcl dialog definition file to be used in           //"
                    "//  conjunction with NumInc.lsp                               //"
                    "//------------------------------------------------------------//"
                    "//  Author: Lee Mac, Copyright  2011 - www.lee-mac.com       //"
                    "//------------------------------------------------------------//"
                    ""
                    "//  --=={ Sub-Assembly Definitions }==--"
                    ""
                    "edit : edit_box     { edit_width = 5; alignment = centered; fixed_width = true; }"
                    "but1 : button       { width = 12;   fixed_width = true; alignment = centered;   }"
                    "but2 : button       { width = 19.5; fixed_width = true; alignment = centered;   }"
                    "rad1 : radio_button { alignment = centered;                                     }"
                    "txt1 : text         { alignment = centered; fixed_width = false;                }"
                    "txt2 : text         { alignment = centered; fixed_width = true; width = 8.8;    }"
                    "txt3 : text         { alignment = left;     fixed_width = false;                }"
                    "btxt : image        { fixed_width = true; height = 1.2; fixed_height = true; color = dialog_background; }"
                    "ctxt : text         { width = 30; fixed_width = true; alignment = centered; }"
                    "pop1 : popup_list   { width = 28; fixed_width = true; }"
                    "spc1 : spacer       { height = 0.1; fixed_height = true; width = 0.1; fixed_width = true; }"
                    ""
                    "//------------------------------------------------------------//"
                    "//                    Main Dialog Definition                  //"
                    "//------------------------------------------------------------//"
                    ""
                    "numinc : dialog { key = \"dcltitle\"; spacer;"
                    "    : row {"
                    "        : column {"
                    "            : toggle { alignment = left; key = \"dyn-flg\"; label = \"Text Follows Cursor\"; }"
                    "        }"
                    "        : column {"
                    "            : text { label = \"Copyright (c) Lee Mac 2011\"; alignment = right; }"
                    "        }"
                    "    }"
                    "    spacer;"
                    "    : row {"
                    "        : column { "
                    "            : boxed_column { label = \"Increment Format\"; width = 37; fixed_width = true;"
                    "                : row { alignment = centered; fixed_width = true;"
                    "                    : column {"
                    "                        : edit { key =  \"pre-str\"; }"
                    "                        : txt2 { label = \"Prefix\"; }"
                    "                    }"
                    "                    : column {"
                    "                        : edit { key =  \"mid-str\"; }"
                    "                        : txt2 { label = \"Middle\"; }"
                    "                    }"
                    "                    : column {"
                    "                        : edit { key =  \"suf-str\"; }"
                    "                        : txt2 { label = \"Suffix\"; }"
                    "                    }"
                    "                }       "
                    "                spacer;"
                    "                : edit { label = \"Increment: \"; key = \"inc-str\"; }"
                    "                spacer;"
                    "                : boxed_column { label = \"Section to Increment\";"
                    "                    : row {"
                    "                        : rad1 { label = \"Prefix\"; key = \"inc-pre\"; }"
                    "                        : rad1 { label = \"Middle\"; key = \"inc-mid\"; }"
                    "                        : rad1 { label = \"Suffix\"; key = \"inc-suf\"; }"
                    "                    }"
                    "                    spacer;"
                    "                }"
                    "                spacer;"
                    "            }"
                    "            : boxed_column { label = \"Border Options\"; spacer;"
                    "                : toggle { key = \"bor-enc\"; label = \"Enclose Text with: \"; }"
                    "                : row {"
                    "                    : popup_list { key = \"bor-shp\"; width = 18; fixed_width = true;  }"
                    "                    : column {"
                    "                        spc1;"
                    "                        : edit_box { edit_width = 5; fixed_width = true; key = \"bor-sid\"; label = \"Sides:\"; }"
                    "                        spc1;"
                    "                    }"
                    "                }"
                    "                : text { label = \"Layer: \"; key = \"bor-ltx\"; }"
                    "                : popup_list { key = \"bor-lay\"; }"
                    "                spacer;"
                    "                : boxed_column { label = \"Border Size\";"
                    "                    : row { fixed_width = true; alignment = centered;"
                    "                        : radio_column {"
                    "                            : radio_button { key = \"bor-off\"; label = \"  Offset:\"; }"
                    "                            : radio_button { key = \"bor-fix\"; label = \"   Fixed:\"; }"
                    "                        }"
                    "                        : column {"
                    "                            : edit { key = \"off-ed1\" ; }"
                    "                            : edit { key = \"fix-ed1\" ; }"
                    "                        }"
                    "                        : column { fixed_width = true;"
                    "                            spacer_1;"
                    "                            : text { label = \" x\"; key = \"fix-txt\"; }"
                    "                        }"
                    "                        : column {"
                    "                            spacer_1;"
                    "                            : edit { key = \"fix-ed2\"; }"
                    "                        }"
                    "                    }"
                    "                    : row { fixed_width = true; alignment = centered;"
                    "                        : spacer { height = 0.1; fixed_height = true; width = 11; fixed_width = true; }"
                    "                        : but2 { key = \"bor-pik\"; label = \">>\"; }"
                    "                    }"
                    "                    spacer;"
                    "                }"
                    "                spacer;"
                    "            }"
                    "        }"
                    "        : column { "
                    "            : boxed_column { label = \"Object\"; width = 37; fixed_width = true;"
                    "                : radio_row { alignment = centered; fixed_width = true;"
                    "                    : rad1 { label = \"Text\";  key = \"obj-txt\"; }"
                    "                    : rad1 { label = \"MText\"; key = \"obj-mtx\"; }"
                    "                    : rad1 { label = \"Block\"; key = \"obj-blk\"; }"
                    "                }"
                    "                spacer;"
                    "                : text { label = \"Block:\"; key = \"blk-txt\"; }"
                    "                : row {"
                    "                    : pop1 { key = \"blk-nme\"; }"
                    "                    : button { label = \">>\"; key = \"blk-pik\"; fixed_width = true;"
                    "                               fixed_height = true; alignment = centered;"
                    "                    }"
                    "                }"
                    "                spacer;"
                    "                : text { label = \"Attribute:\"; key = \"att-txt\"; }"
                    "                : popup_list { key = \"att-nme\"; }"
                    "                spacer;"
                    "            }"
                    "            : boxed_column { label = \"Formatting\"; spacer;"
                    "                : text { label = \"Text Layer: \"; key = \"lay-txt\"; }"
                    "                : popup_list { key = \"txt-lay\"; }"
                    "                spacer;"
                    "                : text { label = \"Text Style: \"; key = \"sty-txt\"; }"
                    "                : popup_list { key = \"txt-sty\"; }"
                    "                spacer;"
                    "                : text { label = \"Text Alignment:\"; key = \"aln-txt\"; }"
                    "                : popup_list { key = \"txt-aln\"; }"
                    "                spacer;"
                    "                : boxed_column { label = \"Text Height\";"
                    "                    : row { fixed_width = true; alignment = centered;"
                    "                        : toggle { key = \"txt-bst\"; label = \"By Style\"; }"
                    "                        : column {"
                    "                            : edit { key = \"txt-sze\"; }"
                    "                            spacer;"
                    "                        }"
                    "                    }"
                    "                    spacer;"
                    "                }"
                    "                spacer;"
                    "            }"
                    "        }"
                    "    }"
                    "    : errtile { width = 34; }"
                    "    spacer;"
                    "    : row {"
                    "        spacer;"
                    "        : but1 { key = \"accept\"; label = \"OK\"; is_default = true; }"
                    "        : but1 { key = \"about\";  label = \"About\"; }"
                    "        : but1 { key = \"cancel\"; label = \"Cancel\"; is_cancel = true; }"
                    "        spacer;"
                    "    }"
                    "    spacer;"
                    "}"
                    ""
                    "//------------------------------------------------------------//"
                    "//                  'About' Dialog Definition                 //"
                    "//------------------------------------------------------------//"
                    ""
                    "about : dialog { label = \"About\"; spacer;"
                    "    : btxt  { key   = \"title1\"; alignment = centered; width = 31.5; }"
                    "    : ctxt  { value = \"Designed and Created by Lee Mac 2011\"; }"
                    "    : button { key = \"weblink\"; label = \"www.lee-mac.com\"; fixed_width = true; alignment = centered;"
                    "               action = \"(startapp \\\"explorer\\\" \\\"http://www.lee-mac.com\\\")\"; }"
                    "    spacer;"
                    "    : row { fixed_width = true; alignment = left;"
                    "        spacer;"
                    "        : btxt  { width = 21.5; key = \"title2\"; alignment = left; }"
                    "    }"
                    "    spacer;"
                    "    : row { fixed_width = true; alignment = centered; spacer;"
                    "        : column {"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "        }"
                    "        : column {"
                    "            : txt1 { label = \"Enter\" ; }"
                    "            : txt1 { label = \"Click\" ; }"
                    "            : txt1 { label = \"<\"     ; }"
                    "            : txt1 { label = \">\"     ; }"
                    "            : txt1 { label = \"O\"     ; }"
                    "            : txt1 { label = \"Tab\"   ; }"
                    "            : txt1 { label = \"M\"     ; }"
                    "            : txt1 { label = \"C\"     ; }"
                    "            : txt1 { label = \"R\"     ; }"
                    "            : txt1 { label = \"T\"     ; }"
                    "            : txt1 { label = \"B\"     ; }"
                    "            : txt1 { label = \"A\"     ; }"
                    "        }"
                    "        : column {"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "        }"
                    "        spacer;"
                    "        : column {"
                    "            : txt3 { label = \"(or Space/Right-Click) Exit Program\"    ; }"
                    "            : txt3 { label = \"Place Object\"                           ; }"
                    "            : txt3 { label = \"Rotate Object Counter-Clockwise\"        ; }"
                    "            : txt3 { label = \"Rotate Object Clockwise\"                ; }"
                    "            : txt3 { label = \"Specify Object Rotation\"                ; }"
                    "            : txt3 { label = \"Rotate Object by 90\"                   ; }"
                    "            : txt3 { label = \"Mirror Object Rotation\"                 ; }"
                    "            : txt3 { label = \"Align Object to Curve\"                  ; }"
                    "            : txt3 { label = \"Replace Existing Text/Attribute String\" ; }"
                    "            : txt3 { label = \"Toggle Increment Counter\"               ; }"
                    "            : txt3 { label = \"Rotate Polygonal Border\"                ; }"
                    "            : txt3 { label = \"Toggle MText Background Mask\"           ; }"
                    "        }"
                    "    }"
                    "    spacer;"
                    "    : row { fixed_width = true; alignment = left;"
                    "        spacer;"
                    "        : btxt  { width = 28; key = \"title3\"; alignment = left; }"
                    "    }"
                    "    spacer;"
                    "    : row { fixed_width = true; alignment = centered; spacer;"
                    "        : column {"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "            : txt1 { label = \"[\"     ; }"
                    "        }"
                    "        : column {"
                    "            : txt1 { label = \"Enter\" ; }"
                    "            : txt1 { label = \"Click\" ; }"
                    "            : txt1 { label = \"+/-\"   ; }"
                    "            : txt1 { label = \"O\"     ; }"
                    "            : txt1 { label = \"P\"     ; }"
                    "            : txt1 { label = \"B\"     ; }"
                    "            : txt1 { label = \"A\"     ; }"
                    "        }"
                    "        : column {"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "            : txt1 { label = \"]\"     ; }"
                    "        }"
                    "        spacer;"
                    "        : column {"
                    "            : txt3 { label = \"(or Space/Right-Click) Exit Alignment\"  ; }"
                    "            : txt3 { label = \"Place Object\"                           ; }"
                    "            : txt3 { label = \"Increase/Decrease Object Offset\"        ; }"
                    "            : txt3 { label = \"Specify Object Offset\"                  ; }"
                    "            : txt3 { label = \"Toggle Object Perpendicularity\"         ; }"
                    "            : txt3 { label = \"Rotate Polygonal Border\"                ; }"
                    "            : txt3 { label = \"Toggle MText Background Mask\"           ; }"
                    "        }"
                    "    }"
                    "    spacer_1; ok_only;"
                    "}"
                    ""
                    "//------------------------------------------------------------//"
                    "//                 End of Dialog Definition                   //"
                    "//------------------------------------------------------------//"
                )
                (write-line line file)
            )
            (setq file (close file))
            (while (not (findfile fname)))
            fname
        )
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:WriteConfig ( name lst / _tostring file )

    (defun _tostring ( x / dimzin )
        (cond
            (   (eq 'INT (type x))
                (itoa x)
            )
            (   (eq 'REAL (type x))
                (setq dimzin (getvar 'DIMZIN))
                (setvar 'DIMZIN 0)
                (setq x (rtos x 2 8))
                (setvar 'DIMZIN dimzin)
                x
            )
            (   (vl-prin1-to-string x)   )
        )
    )
    
    (if (setq file (open name "w"))
        (progn
            (foreach x lst (write-line (_tostring x) file))
            (setq file (close file))
            t
        )
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:ReadConfig ( name lst / file line )
    (if
        (and
            (setq name (findfile name))
            (setq file (open name "r"))
        )
        (progn
            (foreach x lst
                (if (setq line (read-line file))
                    (set x (read line))
                )
            )
            (setq file (close file))
            t
        )
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:Popup ( title flags msg / wsh )
    (vl-catch-all-apply
        (function
            (lambda nil
                (setq wsh (vlax-create-object "WScript.Shell"))
                (vlax-invoke-method wsh 'popup msg 0 title flags)
            )
        )
    )
    (if wsh (vlax-release-object wsh))
    (princ)
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:MakeList ( key lst )
    (start_list key)
    (foreach x lst (add_list x))
    (end_list)
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:TextinCell ( lst pnt str / data dir )
    (setq dir (vlax-3D-point (trans (getvar 'VIEWDIR) 1 0)))
    (if
        (setq data
            (vl-some
                (function
                    (lambda ( table / row col )
                        (if (eq :vlax-true (vla-hittest table pnt dir 'row 'col))
                            (list table row col)
                        )
                    )
                )
                lst
            )
        )
        (not (apply 'vla-settext (append data (list str))))
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:Increment ( sym inc aph )
    (if aph
        (set sym (NumInc:IncrementAlpha (eval sym) (fix (abs (distof inc)))))
        (set sym (NumInc:IncrementNumba (eval sym) inc))
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:IncrementNumba ( str inc / _rtos _decimalplaces incd maxd num slen strd )

    (defun _rtos ( real prec / dimzin result )
        (setq dimzin (getvar 'DIMZIN))
        (setvar 'DIMZIN 0)
        (setq result (rtos real 2 prec))
        (setvar 'DIMZIN dimzin)
        result
    )

    (defun _decimalplaces ( string / pos )
        (if (setq pos (vl-string-position 46 string))
            (- (strlen string) pos 1)
            0
        )
    )
    
    (setq num (+ (distof str) (distof inc)))

    (if (minusp (distof str))
        (setq str (substr str 2))
    )
    (if (minusp (distof inc))
        (setq inc (substr inc 2))
    )
    (setq incd (_decimalplaces inc)
          strd (_decimalplaces str)
          maxd (max incd strd)
          slen (strlen str)
    )
    (cond
        (   (and (< 0 strd) (< 0 incd))
            (setq slen (+ (- slen strd) maxd))
        )
        (   (and (= 0 strd) (< 0 incd))
            (setq slen (+ incd slen 1))
        )
    )
    (setq str (_rtos num maxd))
    (if (minusp num)
        (setq str (substr str 2))
    )
    (while (< (strlen str) slen)
        (setq str (strcat "0" str))
    )
    (if (minusp num)
        (strcat "-" str)
        str
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:IncrementAlpha ( str inc / _IncrementAlpha a )

    (defun _IncrementAlpha ( a b / c d e )
        (cond
            (   (cond
                    (   (< 47 (setq c (car a)) 58)
                        (setq d 48
                              e 10
                        )
                    )
                    (   (< 64 c 91)
                        (setq d 65
                              e 26
                        )
                    )
                    (   (< 96 c 123)
                        (setq d 97
                              e 26
                        )
                    )
                )
                (setq c (+ (- c d) b)
                      b (/ c e)
                )
                (cons (+ d (rem c e))
                    (if (zerop b)
                        (cdr a)
                        (if (cdr a)
                            (_IncrementAlpha (cdr  a) b)
                            (_IncrementAlpha (list d) (if (= 10 e) b (1- b)))
                        )
                    )
                )
            )
            (   (cons c
                    (if (cdr a)
                        (_IncrementAlpha (cdr a) b)
                        (_IncrementAlpha (list 65) (1- b))
                    )
                )
            )
        )
    )

    (vl-list->string
        (reverse
            (if (setq a (reverse (vl-string->list str)))
                (_IncrementAlpha a inc)
                (_IncrementAlpha '(65) (1- inc))
            )
        )
    )
)

;;-------------------------------------------------------------------------------;;
        
(defun NumInc:AligntoCurve ( obj prp ent bor / a1 fac fl g1 g2 gr ll msg mtx p1 ur xa )
    
    (setq fac
        (if (eq "AcDbBlockReference" (vla-get-objectname obj))
            (progn
                (vla-getboundingbox obj 'll 'ur)
                (/
                    (-
                        (cadr (vlax-safearray->list ur))
                        (cadr (vlax-safearray->list ll))
                    )
                    2.0
                )
            )
            (vla-get-height obj)
        )
    )
    
    (setq msg
        (princ
            (strcat
                "\nClick to Align <Exit>: [+/-] for [O]ffset, [P]erpendicular"
                (if (and bor (eq "3" bor-shp)) ", Rotate [B]order" "")
                (if (setq mtx (eq "AcDbMText" (vla-get-objectname obj)))
                    ", B[a]ckground Mask"
                    ""
                )
            )
        )
    )

    (setq xa
        (angle '(0.0 0.0 0.0)
            (trans (getvar 'UCSXDIR) 0 (trans '(0.0 0.0 1.0) 1 0 t))
        )
    )
    
    (while
        (progn (setq gr (grread t 15 0) g1 (car gr) g2 (cadr gr))
            (cond
                (   (member g1 '(5 3))
                    (setq p1 (vlax-curve-getclosestpointto ent (setq g2 (trans g2 1 0)))
                          a1 (angle p1 g2)
                          p1 (vlax-3D-point (polar p1 a1 (* fac crv-off)))
                          a1 (NumInc:MakeReadable (+ a1 crv-per))
                    )
                    (if bor
                        (vla-move bor (vlax-get-property obj prp) p1)
                    )
                    (vlax-put-property obj prp p1)
                    (if bor
                        (vla-rotate bor p1
                            (- a1
                                (if mtx
                                    (+ (vla-get-rotation obj) xa)
                                    (vla-get-rotation obj)
                                )
                            )
                        )
                    )
                    (vla-put-rotation obj ((lambda ( a ) (if mtx (- a xa) a)) a1))
                    (null (setq fl (= g1 3)))
                )
                (   (= 25 g1)
                    nil
                )
                (   (= 02 g1)
                    (cond
                        (   (member g2 '(80 112))  ;; P/p
                            (setq crv-per (- (/ pi 2.0) crv-per))
                        )
                        (   (member g2 '(45 95))   ;; -
                            (setq crv-off (- crv-off 0.1))
                        )
                        (   (member g2 '(43 61))   ;; +
                            (setq crv-off (+ crv-off 0.1))
                        )
                        (   (member g2 '(13 32))   ;; Enter/Space
                            nil
                        )
                        (   (member g2 '(79 111))  ;; O/o
                            (setq crv-off
                                (/
                                    (cond
                                        (   (getdist (strcat "\nSpecify Offset <" (rtos (* fac crv-off)) ">: ")))
                                        (   (* fac crv-off))
                                    )
                                    fac
                                )
                            )
                            (princ msg)
                        )
                        (   (and (member g2 '(65 97)) mtx)  ;; A/a
                            (vlax-put obj 'backgroundfill
                                (setq mtx-bak (~ (vlax-get obj 'backgroundfill)))
                            )
                            (if (zerop mtx-bak)
                                (princ "\n<Background Mask Off>")
                                (princ "\n<Background Mask On>")
                            )
                            (princ msg)
                        )
                        (   (member g2 '(66 98))  ;; B/b
                            (if (and bor (eq "3" bor-shp))
                                (progn
                                    (setq bor-rot (not bor-rot))
                                    (vla-rotate bor
                                        (vlax-3D-point (NumInc:PolygonCentroid bor))
                                        (/ pi bor-sid#)
                                    )
                                )
                                (princ (strcat "\nInvalid Keypress." msg))
                            )
                            t
                        )
                        (   (princ (strcat "\nInvalid Keypress." msg))   )
                    )
                )
            )
        )
    )
    fl  
)

;;-------------------------------------------------------------------------------;

(defun NumInc:MakeReadable ( a )
    (   (lambda ( a ) (if (and (< (/ pi 2.0) a) (<= a (/ (* 3.0 pi) 2.0))) (+ a pi) a))
        (rem (+ a pi pi) (+ pi pi))
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:PolygonCentroid ( obj / _group )

    (defun _group ( lst )
        (if lst
            (cons (list (car lst) (cadr lst)) (_group (cddr lst)))
        )
    )

    (
        (lambda ( lst )
            (
                (lambda ( len )
                    (mapcar '/ (apply 'mapcar (cons '+ lst)) (list len len))
                )
                (length lst)
            )              
        )
        (_group (vlax-get obj 'coordinates))
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:CreateTextBorder ( ent typ off fx1 fx2 sid / cen i inc lst mat pts rad rot vec )
    (cond
        (   (setq lst (NumInc:GetTextBox ent off))
            (setq cen (mapcar '(lambda ( a b ) (/ (+ a b) 2.0)) (car lst) (caddr lst))
                  rot (if (eq "MTEXT" (cdr (assoc 0 (setq ent (entget ent)))))
                          (angle '(0. 0. 0.) (trans (cdr (assoc 11 ent)) 0 (cdr (assoc 210 ent))))
                          (cdr (assoc 50 ent))
                      )
                  ent (cdr (assoc -1 ent))
            )
            (cond
                (   (eq "0" typ) ;; Circle
                    (entmakex
                        (list
                            (cons 0 "CIRCLE")
                            (cons 10 cen)
                            (cons 40 (cond ( fx1 ) ((distance cen (car lst)))))
                            (assoc 210 (entget ent))
                        )
                    )
                )
                (   (member typ '("1" "2"))  ;; Rectangle / Slot
                    (if (and fx1 fx2)
                        (progn
                            (setq fx1 (/ fx1 2.0)
                                  fx2 (/ fx2 2.0)
                            )
                            (setq mat
                                (list
                                    (list (cos rot) (- (sin rot)) 0.0)
                                    (list (sin rot)    (cos rot)  0.0)
                                    (list   0.0           0.0     1.0)
                                )
                            )
                            (setq vec (mapcar '- cen (mxv mat cen)))
                            (setq lst
                                (list
                                    (list (- (car cen) fx1) (- (cadr cen) fx2))
                                    (list (+ (car cen) fx1) (- (cadr cen) fx2))
                                    (list (+ (car cen) fx1) (+ (cadr cen) fx2))
                                    (list (- (car cen) fx1) (+ (cadr cen) fx2))
                                )
                            )
                            (entmakex
                                (append
                                    (list
                                        (cons 0 "LWPOLYLINE")
                                        (cons 100 "AcDbEntity")
                                        (cons 100 "AcDbPolyline")
                                        (cons 90 4)
                                        (cons 70 1)
                                    )
                                    (apply 'append
                                        (mapcar
                                            (function
                                                (lambda ( a b )
                                                    (list (cons 10 (mapcar '+ (mxv mat a) vec)) (cons 42 b))
                                                )
                                            )
                                            lst (if (eq "1" typ) '(0.0 0.0 0.0 0.0) '(0.0 1.0 0.0 1.0))
                                        )
                                    )
                                    (list (assoc 210 (entget ent)))
                                )
                            )
                        )
                        (entmakex
                            (append
                                (list
                                    (cons 0 "LWPOLYLINE")
                                    (cons 100 "AcDbEntity")
                                    (cons 100 "AcDbPolyline")
                                    (cons 90 4)
                                    (cons 70 1)
                                )
                                (apply 'append
                                    (mapcar
                                        (function
                                            (lambda ( a b ) (list (cons 10 a) (cons 42 b)))
                                        )
                                        lst (if (eq "1" typ) '(0.0 0.0 0.0 0.0) '(0.0 1.0 0.0 1.0))
                                    )
                                )
                                (list (assoc 210 (entget ent)))
                            )
                        )
                    )
                )
                (   t ;; Polygon
                    (setq inc (/ (+ pi pi) sid)
                          rad (cond ( fx1 ) ((/ (distance cen (car lst)) (cos (/ inc 2.0)))))
                            i -1
                    )
                    (if (= 1 (logand 1 sid)) (setq rot (+ rot (/ pi 2.))))
                    (repeat sid
                        (setq pts
                            (cons
                                (cons 10
                                    (polar cen (+ rot (* (setq i (1+ i)) inc)) rad)
                                )
                                pts
                            )
                        )
                    )
                    (entmakex
                        (append
                            (list
                                (cons 0 "LWPOLYLINE")
                                (cons 100 "AcDbEntity")
                                (cons 100 "AcDbPolyline")
                                (cons 90 (length pts))
                                (cons 70 1)
                            )
                            (reverse pts)
                            (list (assoc 210 (entget ent)))
                        )
                    )
                )
            )
        )
    )
)

;;-------------------------------------------------------------------------------;;

(defun NumInc:GetTextBox ( e off / dxf a b h l m n o p r w )

    (setq dxf (lambda ( x l ) (cdr (assoc x l))))

    (if
        (setq l
            (cond
                (   (eq "TEXT" (dxf 0 (setq e (entget e))))
                    (setq b (dxf 10 e)
                          r (dxf 50 e)
                    )
                    (
                        (lambda ( a )
                            (mapcar
                                (function
                                    (lambda ( b )
                                        (mapcar
                                            (function
                                                (lambda ( c )
                                                    ((eval (car c)) ((eval (cdr c)) a) off)
                                                )
                                            )
                                            b
                                        )
                                    )
                                )
                                (list
                                    (list (cons '- 'caar ) (cons '- 'cadar ))
                                    (list (cons '+ 'caadr) (cons '- 'cadar ))
                                    (list (cons '+ 'caadr) (cons '+ 'cadadr))
                                    (list (cons '- 'caar ) (cons '+ 'cadadr))
                                )
                            )
                        )
                        (textbox e)
                    )
                )
                (   (eq "MTEXT" (dxf 0 e))
                    (setq n (dxf 210 e)
                          b (trans (dxf 10 e) 0 n)
                          r (angle '(0. 0. 0.) (trans (dxf 11 e) 0 n))
                          w (dxf 42 e)
                          h (dxf 43 e)
                    )
                    (
                        (lambda ( o )
                            (mapcar
                                (function
                                    (lambda ( x ) (mapcar '+ o x))
                                )
                                (list
                                    (list (-   off) (-   off))
                                    (list (+ w off) (-   off))
                                    (list (+ w off) (+ h off))
                                    (list (-   off) (+ h off))
                                )
                            )
                        )
                        (
                            (lambda ( j )
                                (list
                                    (cond
                                      (   (member j '(2 5 8))
                                          (/ w -2.)
                                      )
                                      (   (member j '(3 6 9))
                                          (- w)
                                      )
                                      (   0.0   )
                                    )
                                    (cond
                                      (   (member j '(1 2 3))
                                          (- h)
                                      )
                                      (   (member j '(4 5 6))
                                          (/ h -2.)
                                      )
                                      (   0.0   )
                                    )
                                )
                            )
                            (dxf 71 e)
                        )
                    )
                )
            )
        )
        (progn
            (setq m
                (list
                    (list (cos r) (sin (- r)) 0.)
                    (list (sin r) (cos    r)  0.)
                    (list   0.         0.     1.)
                )
            )
            (mapcar
                (function
                    (lambda ( p )
                        (mapcar '+
                            (mapcar
                                (function
                                    (lambda ( r ) (apply '+ (mapcar '* r p)))
                                )
                                m
                            )
                            (reverse (cdr (reverse b)))
                        )
                    )
                )
                l
            )
        )
    )
)

;; Matrix x Vector - Vladimir Nesterovsky
;; Args: m - nxn matrix, v - vector in R^n

(defun mxv ( m v )
  (mapcar '(lambda ( r ) (apply '+ (mapcar '* r v))) m)
)

;;-------------------------------------------------------------------------------;;

(vl-load-com) (princ)
(princ
    (strcat
        "\n:: NumInc.lsp | Version " NumIncVersion " |  Lee Mac 2011 www.lee-mac.com ::"
        "\n:: Type \"NumInc\" to Invoke ::"
    )
)
(princ)

;;-------------------------------------------------------------------------------;;
;;                                 End of File                                   ;;
;;-------------------------------------------------------------------------------;;